//
// Unit sislt Talariuksen kyttmt kysymysluokat sek kysymysluokkien
// tietorakenteiden toteutuksen. Avoin kysymys ja tietoruutu ovat periaatteessa
// samat, niille on tehty luokat kuitenkin erottelua varten.
// Monivalintakysymysten luokka TMonivalintaKysymys on peritty TTietoruudusta
// ja mukaan on listty toiminnallisuus n-kappaleen vastausvaihtoehdon
// hallitsemiseksi.
//
// Tekijt:
//   Antti Kirjavainen (ajkirjav@iki.fi)
//   Jukka Miettinen (jupemiet@cc.jyu.fi)
//   Heikki Nevala (hemaneva@cc.jyu.fi)
//
// Viimeksi muokattu: 15.7.2005
//

unit KysymysLuokat;

interface

uses SysUtils, Classes;

type

  // Luokka tallettamaan kysymystyyppeihin liittyvien tiedostojen polut
  // Kaikki kysymysluokat peritn tst
  TKysymyskanta = Class
    private
      FVideonPolku: string;
      FAanenPolku: string;
      FKuvanPolku: string;
    public
    published
      property KuvanPolku : string read FKuvanPolku write FKuvanPolku;
      property AanenPolku : string read FAanenPolku write FAanenPolku;
      property VideonPolku : string read FVideonPolku write FVideonPolku;
  end;

  // Luokka tallettamaan tekstimuotoista dataa tietoruutujen yhteydess.
  TTietoruutu = Class(TKysymyskanta)
    private
      FTeksti : string; // Tietoruudun teksti tai mahdollisesti kysymys tms.
    public
      Constructor Create();
      Destructor Destroy; Override;
      function Copy(): TTietoruutu;
    published
      property TietoTeksti : string read FTeksti write Fteksti;
  end;

  // Luokka tallettamaan tekstimuotoista dataa avointen kysymysten yhteydess.
  // Periaatteessa sama kuin edellinen, mutta tehdn erottelun vuoksi.
  TAvoinKysymys = Class(TKysymyskanta)
    private
      FTeksti : string; // Tietoruudun teksti tai mahdollisesti kysymys tms.
    public
      Constructor Create();
      Destructor Destroy; Override;
      function Copy(): TAvoinKysymys; overload;
    published
      property Kysymys : string read FTeksti write Fteksti;
  end;

  // Luokka tallettamaan dataa monivalintakysymysten yhteydess.
  TMonivalintaKysymys = Class(TAvoinKysymys)
    private
      FVaaratVastaukset : TStringlist; // Vrien vastausten tekstit
      FOikeaVastaus : string;
      FVastauksenKuvapolku : TStringlist; // Vastauksiin liitettvien kuvien
                                          // polut
    public
      Constructor Create();
      Destructor Destroy; Override;
      function Copy(): TMonivalintaKysymys; overload;
    published
      property VaaratVastaukset : TStringList read FVaaratVastaukset write
                                  FVaaratVastaukset;
      property VastauksenKuvapolku : TStringList read FVastauksenKuvapolku write
                                     FVastauksenKuvapolku;

      property OikeaVastaus : string read FOikeaVastaus write FOikeaVastaus;
  end;



implementation

{ TMonivalintaKysymys -------------------------------------------------------- }

function TMonivalintaKysymys.Copy: TMonivalintaKysymys;
var
  i : integer;
begin
  Result := TMonivalintaKysymys.Create();
  Result.Kysymys := self.Kysymys;
  Result.OikeaVastaus := OikeaVastaus;
  Result.KuvanPolku := self.KuvanPolku;
  Result.AanenPolku := self.AanenPolku;
  Result.VideonPolku := self.VideonPolku;
  for i:=0 to VaaratVastaukset.Count - 1 do begin
    Result.VaaratVastaukset.Add(VaaratVastaukset[i]);
  end;
  for i:=0 to VastauksenKuvaPolku.Count-1 do begin
    Result.VastauksenKuvapolku.Add(VastauksenKuvapolku[i]);
  end;
end;

constructor TMonivalintaKysymys.Create;
begin
  inherited;
  // Luodaan lista vrille vastauksille
  VaaratVastaukset := TStringList.Create;
  // Luodaan lista vastausvaihtoehtojen kuvapoluille
  VastauksenKuvapolku:=TStringlist.Create;
  OikeaVastaus:='';
end;

destructor TMonivalintaKysymys.Destroy;
begin
  // Tuhotaan vrien vastausten lista (tss oletetaan, ettei listassa
  // ole viitteit olioihin, vain teksti string-muodossa)
  VaaratVastaukset.Free;
  VastauksenKuvapolku.Free;
  inherited;
end;

{ TTietoruutu ---------------------------------------------------------------- }

function TTietoruutu.Copy: TTietoruutu;
begin
  Result := TTietoruutu.Create();
  Result.TietoTeksti := self.TietoTeksti;
  Result.KuvanPolku := self.KuvanPolku;
  Result.AanenPolku := self.AanenPolku;
  Result.VideonPolku := self.VideonPolku;
end;

constructor TTietoruutu.Create;
begin
  TietoTeksti:='';
end;

destructor TTietoruutu.Destroy;
begin
  TietoTeksti:='';
  inherited;
end;

{ TAvoinKysymys }

function TAvoinKysymys.Copy: TAvoinKysymys;
begin
  Result := TAvoinKysymys.Create();
  Result.Kysymys := self.Kysymys;
  Result.KuvanPolku := self.KuvanPolku;
  Result.AanenPolku := self.AanenPolku;
  Result.VideonPolku := self.VideonPolku;
end;

constructor TAvoinKysymys.Create;
begin
  Kysymys:='';
end;

destructor TAvoinKysymys.Destroy;
begin
  Kysymys:='';
  inherited;
end;

end.
