unit Paketit;

interface

uses Contnrs, Classes, md5, XMLDoc, XMLIntf, xmldom, StrUtils;

//pakettien tyyppien vakiot
{$MINENUMSIZE 1} //laitetaan yksittisen arvon kooksi 1 tavu, joten eri arvoja voi olla maksimissaan 256
type Pakettityyppi =
  (PingTyyppi,
  AnnaChattaajaListaTyyppi,
  ChattaajaListaTyyppi,
  ChatViestiTyyppi,
  ChatViestiLahettajaltaTyyppi,
  TunnuksenLuominenTyyppi,
  YhdistysTyyppi,
  KirjauduTyyppi,
  KirjautuminenTyyppi,
  KirjautunutTyyppi,
  ListanPaivitysTyyppi,
  PelienMaaraTyyppi,
  PelilistaTyyppi,
  UusiPeliTyyppi,
  AnnaPelinTarkemmatTiedotTyyppi,
  PelinTarkemmatTiedotTyyppi,
  AnnaPelaajalistaTyyppi,
  PelaajalistaTyyppi,
  PelinTiedostotTyyppi,
  LiityPeliinTyyppi,
  LiityitPeliinTyyppi,
  LiittyiPeliinTyyppi,
  PoistuTyyppi,
  PoistuiTyyppi,
  PeliAlkaaTyyppi,
  KenenVuoroTyyppi,
  HeitaTyyppi,
  HeitonTulosTyyppi,
  SiirrettyTyyppi,
  OdotaTyyppi,
  KysymysTyyppi,
  VastausTyyppi,
  TulosTyyppi,
  PeliPaattyiTyyppi,
  ArvosteluTyyppi,
  PeliPaattynytTyyppi,
  KatkaiseYhteysTyyppi,
  VirhePakettiTyyppi,
  KirjaaTiedostopalvelimeksiTyyppi,
  AnnaTunnuslistaTyyppi,
  TunnuslistaTyyppi,
  LisattyTunnusTyyppi,
  AnnaTiedostopalvelimenTiedotTyyppi,
  TiedostopalvelimenTiedotTyyppi,
  LokitiedostoTyyppi);

//virhekoodit
{$MINENUMSIZE 2} //laitetaan yksittisen arvon kooksi 2 tavu, joten eri arvoja voi olla maksimissaan 65536
type Virhekoodi =
  (EiVirhettaVirhe,
  KayttajatunnusJoKaytossaVirhe,
  PeliaJoMaksimimaaraVirhe,
  PelinNimiJoKaytossaVirhe,
  VaaraMetapalvelimenSalasanaVirhe);

{$MINENUMSIZE 1} //laitetaan yksittisen arvon kooksi 1 tavu, joten eri arvoja voi olla maksimissaan 256

type Reaktiokoodi =
  (OikeaTulosReaktio,
  VaaraTulosReaktio,
  UusiHeittoReaktio);

{$MINENUMSIZE 1} //laitetaan yksittisen arvon kooksi 1 tavu, joten eri arvoja voi olla maksimissaan 256

type Liittymiskoodi =
  (HyvaksyttyIlmoitus,
  VaaraSalasanaIlmoitus,
  PeliJoTaynnaIlmoitus);

{$MINENUMSIZE 1} //laitetaan oletusarvo takaisin

//XML-dokumenteiss kytettvt vakiot
const XMLDokumentinDOM: String = 'MSXML';
const XMLDokumentinSisennys: String = '    ';
const XMLDokumentinVersio: String = '1.0';
const XMLDokumentinMerkkienKoodaus: String = 'ISO-8859-1';

type
  TMPeli = class
    PelinLuojanNimi: String;
    PelinLuojanTunnus: String;
    PelinNimi: String;
    Aihealue: String;
    Vaikeustaso: Byte;
    PelipalvelimenOsoite: String;
    Kaynnissa: Boolean;
  public
    constructor Create(KLuojanNimi: String; KPelinLuojanTunnus: String; KNimi: String; KAihealue: String; KVaikeustaso: Byte; KPelipalvelimenOsoite: String; KKaynnissa: Boolean);
  end;

  TPaketti = class
    XMLTulkki: IXMLDocument;
    //PakettiTaulukko: array of Byte;
    //PakettiTaulukonPituus: Integer;
    //PaketinPituus: Integer;
    Paketti: String;
    PaketinTyyppi: Pakettityyppi;

    function VastaanotettavanMuodostaja(XML: String): IXMLNode;
    function LahetettavanMuodostaja(Tyyppi: Pakettityyppi): IXMLNode;
    destructor Destroy; override;
  public
    constructor Create(XML: String); overload; virtual;

    function PalautaPaketti: String;
    function PalautaTyyppi: Pakettityyppi;
  end;

  TPing = class(Tpaketti)
  public
    constructor Create(XML: String); override;
    constructor Create; overload;
  end;

  TAnnaChattaajaLista = class(TPaketti)
  public
    constructor Create(XML: String); override;
    constructor Create(); overload;
  end;

  TChattaajaLista = class(TPaketti)
    PelaajienNimet: TStringList;
    destructor Destroy; override;
  public
    constructor Create(XML: String); override;
    constructor Create(KPelaajienNimet: TStringList); overload;
  end;

  TChatViesti = class(TPaketti)
    Sisalto: String;
  public
    constructor Create(XML: String); override;
    constructor Create(KSisalto: String; Lahetettava: Boolean); overload;
  end;

  TChatViestiLahettajalta = class(TPaketti)
    Sisalto: String;
  public
    constructor Create(XML: String); override;
    constructor Create(KSisalto: String; Lahetettava: Boolean); overload;
  end;

  TTunnuksenLuominen = class(TPaketti)
    PelaajanNimi: String;
    Tunnus: String;
    SalasananMD5: MD5Digest;
    MetapalvelimenSalasananMD5: MD5Digest;
  public
    constructor Create(XML: String); override;
    constructor Create(KPelaajanNimi: String; KTunnus: String; KSalasananMD5: MD5Digest; KMetapalvelimenSalasananMD5: MD5Digest); overload;
  end;

  TYhdistys = class(TPaketti)
  public
    constructor Create(XML: String); override;
    constructor Create; overload;
  end;

  TKirjaudu = class(TPaketti)
  public
    constructor Create(XML: String); override;
    constructor Create; overload;
  end;

  TKirjautuminen = class(TPaketti)
    Kayttajatunnus: String;
    SalasanaMD5: MD5Digest;
  public
    constructor Create(XML: String); override;
    constructor Create(KTunnus: String; KSalasanaMD5: MD5Digest); overload;
  end;

  TKirjautunut = class(TPaketti)
    PelaajanNimi: String;
    Hyvaksytty: Boolean;
  public
    constructor Create(XML: String); override;
    constructor Create(KPelaajanNimi: String; KHyvaksytty: Boolean); overload;
  end;

  TListanPaivitys = class(Tpaketti)
  public
    constructor Create(XML: String); override;
    constructor Create; overload;
  end;

  TPelienMaara = class(TPaketti)
    Maara: Word;
  public
    constructor Create(XML: String); override;
    constructor Create(KMaara: Word); overload;
  end;

  TPelilista = class(TPaketti)
    AloitusIndeksi: Word;
    Pelit: TObjectList;
    destructor Destroy; override;
  public
    constructor Create(XML: String); override;
    constructor Create(KAloitusIndeksi: Word; KPelit: TObjectList); overload;
  end;

  TUusiPeli = class(TPaketti)
    Pelinnimi: String;
    Aihealue: String;
    Vaikeustaso: Byte;
  public
    constructor Create(XML: String); override;
    constructor Create(KPelinnimi: String; KAihealue: String; KVaikeustaso: Byte); overload;
  end;

  TAnnaPelinTarkemmatTiedot = class(TPaketti)
  public
    constructor Create(XML: String); override;
    constructor Create; overload;
  end;

  TPelinTarkemmatTiedot = class(TPaketti)
    MaksimiPelaajaMaara, PelaajienMaara,
    Arvosana: Integer;
    ArvosananKysyminen: Boolean;
    SalasanallaSuojattu: Boolean;
    PelaajienNimet: TStringList;
    PelaajienTunnukset: TStringList;
    PelinNimi: String;
    destructor Destroy; override;
  public
    constructor Create(XML: String); override;
    constructor Create(KMaksimiPelaajaMaara: Byte; KArvosana: Byte;
                       KArvosananKysyminen: Boolean; KSalasanallaSuojattu: Boolean;
                       KPelaajienNimet: TStringList; KPelaajienTunnukset: TStringList;
                       KPelinNimi: String); overload;
  end;

  TAnnaPelaajalista = class(TPaketti)
  public
    constructor Create(XML: String); override;
    constructor Create; overload;
  end;

  TPelaajalista = class(TPaketti)
    Pelaajat: TStringList;
    Tunnukset: TStringList;
    destructor Destroy; override;
  public
    constructor Create(XML: String); override;
    constructor Create(KPelaajat: TStringList; KTunnukset: TStringList); overload;
  end;

  TPelinTiedostot = class(TPaketti)
    Pelilauta: String;
    Kysymyssarjat: TStringList;
    destructor Destroy; override;
  public
    constructor Create(XML: String); override;
    constructor Create(KPelilauta: String; KKysymyssarja: TStringList); overload;
  end;

  TLiityPeliin = class(TPaketti)
    PelaajanNimi: String;
    Tunnus: String;
    SalasanaMD5: MD5Digest;
  public
    constructor Create(XML: String); override;
    constructor Create(KPelaajanNimi: String; KTunnus: String; KSalasanaMD5: MD5Digest); overload;
  end;

  TLiityitPeliin = class(TPaketti)
    Hyvaksytty: Liittymiskoodi;
  public
    constructor Create(XML: String); override;
    constructor Create(KHyvaksytty: Liittymiskoodi); overload;
  end;

 TLiittyiPeliin = class(TPaketti)
    PelaajanNimi: String;
    Tunnus: String;
  public
    constructor Create(XML: String); override;
    constructor Create(KPelaajanNimi: String; KTunnus: String); overload;
  end;

  TPoistu = class(TPaketti)
  public
    constructor Create(XML: String); override;
    constructor Create; overload;
  end;

  TPoistui = class(TPaketti)
    PelaajanNimi: String;
    Tunnus: String;
  public
    constructor Create(XML: String); override;
    constructor Create(KPelaajanNimi: String; KTunnus: String); overload;
  end;

  TPeliAlkaa = class(TPaketti)
  public
    constructor Create(XML: String); override;
    constructor Create; overload;
  end;

  TKenenVuoro = class(TPaketti)
    PelaajanNumero: Byte;
  public
    constructor Create(XML: String); override;
    constructor Create(KPelaajanNumero: Byte); overload;
  end;

  THeita = class(TPaketti)
  public
    constructor Create(XML: String); override;
    constructor Create; overload;
  end;

  THeitonTulos = class(TPaketti)
    HeitonTulos: Byte;
    Siirtyma: Shortint;
  public
    constructor Create(XML: String); override;
    constructor Create(KHeitonTulos: Byte; KSiirtyma: Shortint); overload;
  end;

  TSiirretty = class(TPaketti)
    Siirtyma: Shortint;
  public
    constructor Create(XML: String); override;
    constructor Create(KSiirtyma: Shortint); overload;
  end;

  TOdota = class(TPaketti)
  public
    constructor Create(XML: String); override;
    constructor Create; overload;
  end;

  TKysymys = class(TPaketti)
    KysymyssarjanNumero: Byte;
    KysymyksenNumero: Word;
  public
    constructor Create(XML: String); override;
    constructor Create(KKysymyssarjanNumero: Byte; KKysymyksenNumero: Word); overload;
  end;

  TVastaus = class(TPaketti)
    Vastaus: String;
  public
    constructor Create(XML: String); override;
    constructor Create(KVastaus: String; Lahetettava: Boolean); overload;
  end;

  TTulos = class(TPaketti)
    Reaktio: Reaktiokoodi;
  public
    constructor Create(XML: String); override;
    constructor Create(KReaktio: Reaktiokoodi); overload;
  end;

  TPeliPaattyi = class(TPaketti)
    PelaajienTunnukset: TStringList;
    destructor Destroy; override;
  public
    constructor Create(XML: String); override;
    constructor Create(KPelaajienTunnukset: TStringList); overload;
  end;

  TArvostelu = class(TPaketti)
    Arvosana: Byte;
  public
    constructor Create(XML: String); override;
    constructor Create(KArvosana: Byte); overload;
  end;

  TPeliPaattynyt = class(TPaketti)
    Tilanne: Byte;
    LoginNimi: String; //pit olla pelin nimi
    PelilaudanNimi: String;
    KysymyssarjojenNimet: TStringList;
    ArvosanaTunnukset: TStringList;
    Arvosanat: TStringList;
    destructor Destroy; override;
  public
    constructor Create(XML: String); override;
    constructor Create(KTilanne: Byte; KLoginNimi: String; KPelilaudanNimi: String;
                       KKysymyssarjojenNimet: TStringList; KArvosanaTunnukset: TStringList;
                       KArvosanat: TStringList); overload;
  end;

  TKatkaiseYhteys = class(TPaketti) //asiakas lhett poistuessaan metapalvelimelta
  public
    constructor Create(XML: String); override;
    constructor Create; overload;
  end;

  TVirhePaketti = class(TPaketti)
    Virhe: Virhekoodi;
  public
    constructor Create(XML: String); override;
    constructor Create(KVirhe: Virhekoodi); overload;
  end;

  TKirjaaTiedostopalvelimeksi = class(TPaketti)
  public
    constructor Create(XML: String); override;
    constructor Create; overload;
  end;

  TAnnaTunnusLista = class(TPaketti)
  public
    constructor Create(XML: String); override;
    constructor Create; overload;
  end;

  TTunnusLista = class(TPaketti)
    Tunnukset: TStringList;
    SalasanatMD5: TStringList;
    destructor Destroy; override;
  public
    constructor Create(XML: String); override;
    constructor Create(KTunnukset: TStringList; KSalasanatMD5: TStringList); overload;
  end;

  TLisattyTunnus = class(TPaketti)
    Tunnus: String;
    SalasanaMD5: String;
  public
    constructor Create(XML: String); override;
    constructor Create(KTunnus: String; KSalasanaMD5: String); overload;
  end;

  TAnnaTiedostopalvelimenTiedot = class(TPaketti)
  public
    constructor Create(XML: String); override;
    constructor Create; overload;
  end;

  TTiedostopalvelimenTiedot = class(TPaketti)
    Osoite: String;
    Portti: Integer;
  public
    constructor Create(XML: String); override;
    constructor Create(KOsoite: String; KPortti: Integer); overload;
  end;

//kokovakiot
//const PaketinKoonKoko = 2;
//const TyypinKoko = 1;
//const TekstinPituudenKoko = 1;

const MD5MerkkijononPituus = 32;
const MD5Koko = 16;

const OsoitteenMaksimipituus = 120;
const PelienMaksimimaara = 1000;
const KysymyssarjojenMaksimimaara = 10;
const AihelaueenMaksimipituus = 20;
const PelinNimenMaksimipituus = 50;
const KayttajatunnuksenMaksimipituus = 20;
const KayttajatunnuksenMinimipituus = 4;
const SalasananMaksimipituus = 20;
const SalasananMinimipituus = 4;
const PelaajanNimenMaksimipituus = 60;
const LahetettavienPelienMaksimimaara = 20;
const VastauksenMaksimipituus = 250;
const ChatViestiMaksimipituus = 200;
const PelilaudanMaksimipituus = 100;

//apufunktio, mik muuttaa merkkijonon MD5:ksi
function StringToMD5(MD5String: String): MD5Digest;
//palauttaa paketin tyypin
function PalautaPaketinTyyppi(XML: String): Pakettityyppi;
//erottelee kasautuneet XML-paketit
function PilkoXML(Pilkottava: String): TStringList;

implementation

uses Windows, SysUtils;

function StringToMD5(MD5String: String): MD5Digest;
var MD5: MD5Digest;
    I, J, C1, C2: Integer;
    C: Char;
begin
  //tarkistetaan, ett merkkijono on oikein pituinen
  if (Length(MD5String) <> MD5MerkkijononPituus) then
    raise EConvertError.Create('MD5-tarkistussumma ei ole 32 merkki pitk!');

  //tytetn MD5:n sislt merkkijonon mukaiseksi 
  for I := 0 to MD5Koko - 1 do
  begin
    J := (I) * 2;

    //muunnetaan kerrallaan kaksi heksamerkki, mitk voi olla 0-9 tai a-f, niit vastaavaksi numeerisiksi arvoksi
    C := MD5String[J+1];
    case C of
      '0'..'9': C1 := Ord(C) - Ord('0');
      'a'..'f': C1 := Ord(C) - Ord('a') + 10;
    else
      raise EConvertError.Create('MD5-tarkistussummassa on kiellettyj merkkej!');
    end;

    C := MD5String[J + 2];
    case C of
      '0'..'9': C2 := Ord(C) - Ord('0');
      'a'..'f': C2 := Ord(C) - Ord('a') + 10;
    else
      raise EConvertError.Create('MD5-tarkistussummassa on kiellettyj merkkej!');
    end;

    MD5[I] := C1 * 16 + C2; //lasketaan merkkien mrm arvo 
  end;

  Result := MD5;
end;

//palautetaan paketin tyyppi XML-dokumentin sislt
function PalautaPaketinTyyppi(XML: String): Pakettityyppi;
var XMLTulkki: IXMLDocument;
begin
  XMLTulkki := TXMLDocument.Create(nil);
  XMLTulkki.LoadFromXML(XML);
  XMLTulkki.Active := True;
  Result := Pakettityyppi(StrToInt(XMLTulkki.DocumentElement.ChildNodes['tyyppi'].Text));
  XMLTulkki.Active := False;
end;

//erotetaan yhdess merkkijonossa olevat perkkiset XML-dokumentit TStringList:ksi 
function PilkoXML(Pilkottava: String): TStringList;
var S, E: Integer;
begin
  Result := TStringList.Create;
  E := 0;
  repeat
    S := E+1;
    E := PosEx('<?',Pilkottava, S+2)-1;
    if (E <= 0) then break;
    Result.Add(MidStr(Pilkottava, S, E-S));
  until(E <= 0);
  Result.Add(MidStr(Pilkottava, S, Length(Pilkottava)-S));
end;

constructor TMPeli.Create(KLuojanNimi: String; KPelinLuojanTunnus: String; KNimi: String; KAihealue: String; KVaikeustaso: Byte; KPelipalvelimenOsoite: String; KKaynnissa: Boolean);
begin
  PelinLuojanNimi := KLuojanNimi;
  PelinLuojanTunnus := KPelinLuojanTunnus;
  PelinNimi := KNimi;
  Aihealue := KAihealue;
  Vaikeustaso := KVaikeustaso;
  Kaynnissa := False;
  PelipalvelimenOsoite := KPelipalvelimenOsoite;
  Kaynnissa := KKaynnissa;
end;

destructor TPaketti.Destroy;
begin
  XMLTulkki.Active := False;

  inherited;
end;

function TPaketti.PalautaPaketti: String;
begin
  Result := Paketti;
end;

function TPaketti.PalautaTyyppi: Pakettityyppi;
begin
  Result := PaketinTyyppi;
end;

function TPaketti.VastaanotettavanMuodostaja(XML: String): IXMLNode;
var JuuriNode: IXMLNode;
begin
  XMLTulkki := TXMLDocument.Create(nil);
  XMLTulkki.Active := True;
  XMLTulkki.LoadFromXML(XML);

  JuuriNode := XMLTulkki.DocumentElement;

  PaketinTyyppi := Pakettityyppi(StrToInt(JuuriNode.ChildNodes['tyyppi'].Text));

  Result := JuuriNode;
end;

function TPaketti.LahetettavanMuodostaja(Tyyppi: Pakettityyppi): IXMLNode;
var TyyppiNode: IXMLNode;
    JuuriNode: IXMLNode;
begin
  XMLTulkki := TXMLDocument.Create(nil);
  XMLTulkki.Active := True;
  XMLTulkki.Version := '1.0';
  XMLTulkki.Encoding := 'ISO-8859-1';

  JuuriNode := XMLTulkki.AddChild('paketti');

  TyyppiNode := JuuriNode.AddChild('tyyppi');
  TyyppiNode.Text := IntToStr(Byte(Tyyppi));

  PaketinTyyppi := Tyyppi;

  Result := JuuriNode;
end;

constructor TPing.Create(XML: String);
begin
  VastaanotettavanMuodostaja(XML);
end;

constructor TPing.Create;
begin
  LahetettavanMuodostaja(PingTyyppi);
  XMLTulkki.SaveToXML(Paketti);
end;

constructor TChatViesti.Create(XML: String);
var JuuriNode: IXMLNode;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);

  Sisalto := JuuriNode.ChildNodes['viesti'].Text;
end;

constructor TChatViesti.Create(KSisalto: String; Lahetettava: Boolean);
var JuuriNode: IXMLNode;
    ViestiNode: IXMLNode;
begin
  JuuriNode := LahetettavanMuodostaja(ChatViestiTyyppi);

  ViestiNode := JuuriNode.AddChild('viesti');
  ViestiNode.Text := KSisalto;

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TChatViestiLahettajalta.Create(XML: String);
var JuuriNode: IXMLNode;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);

  Sisalto := JuuriNode.ChildNodes['sisalto'].Text;
end;

constructor TChatViestiLahettajalta.Create(KSisalto: String; Lahetettava: Boolean);
var JuuriNode, ApuNode: IXMLNode;
begin
  JuuriNode := LahetettavanMuodostaja(ChatViestiLahettajaltaTyyppi);

  ApuNode := JuuriNode.AddChild('sisalto');
  ApuNode.Text := KSisalto;

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TTunnuksenLuominen.Create(XML: String);
var JuuriNode: IXMLNode;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);

  PelaajanNimi := JuuriNode.ChildNodes['pelaajannimi'].Text;
  Tunnus := JuuriNode.ChildNodes['tunnus'].Text;
  SalasananMD5 := StringToMD5(JuuriNode.ChildNodes['salasana'].Text);
  MetapalvelimenSalasananMD5 := StringToMD5(JuuriNode.ChildNodes['metasalasana'].Text);
end;

constructor TTunnuksenLuominen.Create(KPelaajanNimi: String; KTunnus: String; KSalasananMD5: MD5Digest; KMetapalvelimenSalasananMD5: MD5Digest);
var JuuriNode, ApuNode: IXMLNode;
begin
  JuuriNode := LahetettavanMuodostaja(TunnuksenLuominenTyyppi);

  ApuNode := JuuriNode.AddChild('pelaajannimi');
  ApuNode.Text := KPelaajanNimi;
  ApuNode := JuuriNode.AddChild('tunnus');
  ApuNode.Text := KTunnus;
  ApuNode := JuuriNode.AddChild('salasana');
  ApuNode.Text := MD5Print(KSalasananMD5);
  ApuNode := JuuriNode.AddChild('metasalasana');
  ApuNode.Text := MD5Print(KMetapalvelimenSalasananMD5);

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TYhdistys.Create(XML: String);
begin
  VastaanotettavanMuodostaja(XML);
end;

constructor TYhdistys.Create;
begin
  LahetettavanMuodostaja(YhdistysTyyppi);

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TKirjaudu.Create(XML: String);
begin
  VastaanotettavanMuodostaja(XML);
end;

constructor TKirjaudu.Create;
begin
  LahetettavanMuodostaja(KirjauduTyyppi);

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TKirjautuminen.Create(XML: String);
var JuuriNode: IXMLNode;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);

  Kayttajatunnus := JuuriNode.ChildNodes['tunnus'].Text;
  SalasanaMD5 := StringToMD5(JuuriNode.ChildNodes['salasana'].Text);
end;

constructor TKirjautuminen.Create(KTunnus: String; KSalasanaMD5: MD5Digest);
var JuuriNode, ApuNode: IXMLNode;
begin
  JuuriNode := LahetettavanMuodostaja(KirjautuminenTyyppi);

  ApuNode := JuuriNode.AddChild('tunnus');
  ApuNode.Text := KTunnus;
  ApuNode := JuuriNode.AddChild('salasana');
  ApuNode.Text := MD5Print(KSalasanaMD5);

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TKirjautunut.Create(XML: String);
var JuuriNode: IXMLNode;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);

  PelaajanNimi := JuuriNode.ChildNodes['pelaajannimi'].Text;
  if(JuuriNode.ChildNodes['hyvaksytty'].Text = '1') then
    Hyvaksytty := True
  else
    Hyvaksytty := False;
end;

constructor TKirjautunut.Create(KPelaajanNimi: String; KHyvaksytty: Boolean);
var JuuriNode, ApuNode: IXMLNode;
begin
  JuuriNode := LahetettavanMuodostaja(KirjautunutTyyppi);

  ApuNode := JuuriNode.AddChild('pelaajannimi');
  ApuNode.Text := KPelaajanNimi;
  ApuNode := JuuriNode.AddChild('hyvaksytty');
  if(KHyvaksytty) then
    ApuNode.Text := '1'
  else
    ApuNode.Text := '0';

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TListanPaivitys.Create(XML: String);
begin
  VastaanotettavanMuodostaja(XML);
end;

constructor TListanPaivitys.Create;
begin
  LahetettavanMuodostaja(ListanPaivitysTyyppi);

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TPelienMaara.Create(XML: String);
var JuuriNode: IXMLNode;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);

  Maara := Word(StrToInt(JuuriNode.ChildNodes['maara'].Text));
end;

constructor TPelienMaara.Create(KMaara: Word);
var JuuriNode, ApuNode: IXMLNode;
begin
  JuuriNode := LahetettavanMuodostaja(PelienMaaraTyyppi); //2 on maara-muuttujan koko

  ApuNode := JuuriNode.AddChild('maara');
  ApuNode.Text := IntToStr(KMaara);

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TPelilista.Create(XML: String);
var JuuriNode, PelitNode, PeliNode: IXMLNode;
    Maara, I: Integer;
    PelinLuojanNimi, PelinLuojanTunnus, PelinNimi,
    Aihealue, PelipalvelimenOsoite: String;
    Vaikeustaso: Byte;
    Kaynnissa: Boolean;

begin
  JuuriNode := VastaanotettavanMuodostaja(XML);

  AloitusIndeksi := Word(StrToInt(JuuriNode.ChildNodes['aloitusindeksi'].Text));
  PelitNode := JuuriNode.ChildNodes['pelit'];
  Maara := PelitNode.ChildNodes.Count;
  Pelit := TObjectList.Create;
  for I := 0 to Maara - 1 do
  begin
    PeliNode := PelitNode.ChildNodes.Get(I);
    PelinLuojanNimi := PeliNode.ChildNodes['pelinluoja'].Text;
    PelinLuojanTunnus := PeliNode.ChildNodes['luojantunnus'].Text;
    PelinNimi := PeliNode.ChildNodes['pelinnimi'].Text;
    Aihealue := PeliNode.ChildNodes['aihealue'].Text;
    PelipalvelimenOsoite := PeliNode.ChildNodes['palvelimenosoite'].Text;
    Vaikeustaso := Byte(StrToInt(PeliNode.ChildNodes['vaikeustaso'].Text));
    Kaynnissa := StrToInt(PeliNode.ChildNodes['kaynnissa'].Text) <> 0;
    Pelit.Add(TMPeli.Create(PelinLuojanNimi, PelinLuojanTunnus, PelinNimi,
                            Aihealue, Vaikeustaso, PelipalvelimenOsoite, Kaynnissa));
  end;
end;

constructor TPelilista.Create(KAloitusIndeksi: Word; KPelit: TObjectList);
var JuuriNode, ApuNode, PelitNode, PeliNode: IXMLNode;
    I: Integer;
    Peli: TMPeli;
begin
  JuuriNode := LahetettavanMuodostaja(PelilistaTyyppi);

  ApuNode := JuuriNode.AddChild('aloitusindeksi');
  ApuNode.Text := IntToStr(KAloitusIndeksi);

  PelitNode := JuuriNode.AddChild('pelit');
  for I := 0 to KPelit.Count - 1 do
  begin
    Peli := KPelit[I] as TMPeli;
    PeliNode := PelitNode.AddChild('peli');
    ApuNode := PeliNode.AddChild('pelinluoja');
    ApuNode.Text := Peli.PelinLuojanNimi;
    ApuNode := PeliNode.AddChild('luojantunnus');
    ApuNode.Text := Peli.PelinLuojanTunnus;
    ApuNode := PeliNode.AddChild('pelinnimi');
    ApuNode.Text := Peli.PelinNimi;
    ApuNode := PeliNode.AddChild('aihealue');
    ApuNode.Text := Peli.Aihealue;
    ApuNode := PeliNode.AddChild('palvelimenosoite');
    ApuNode.Text := Peli.PelipalvelimenOsoite;
    ApuNode := PeliNode.AddChild('vaikeustaso');
    ApuNode.Text := IntToStr(Peli.Vaikeustaso);
    ApuNode := PeliNode.AddChild('kaynnissa');
    ApuNode.Text := IntToStr(Integer(Peli.Kaynnissa));
  end;

  XMLTulkki.SaveToXML(Paketti);
end;

destructor TPelilista.Destroy;
begin
  Pelit.Free;

  inherited;
end;

constructor TUusiPeli.Create(XML: String);
var JuuriNode: IXMLNode;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);

  Aihealue := JuuriNode.ChildNodes['aihealue'].Text;
  Pelinnimi := JuuriNode.ChildNodes['pelinnimi'].Text;
  Vaikeustaso := Byte(StrToInt(JuuriNode.ChildNodes['vaikeustaso'].Text));
end;

constructor TUusiPeli.Create(KPelinnimi: String; KAihealue: String; KVaikeustaso: Byte);
var JuuriNode, ApuNode: IXMLNode;
begin
  JuuriNode := LahetettavanMuodostaja(UusiPeliTyyppi);

  ApuNode := JuuriNode.AddChild('aihealue');
  ApuNode.Text := KAihealue;
  ApuNode := JuuriNode.AddChild('pelinnimi');
  ApuNode.Text := KPelinnimi;
  ApuNode := JuuriNode.AddChild('vaikeustaso');
  ApuNode.Text := IntToStr(KVaikeustaso);

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TAnnaPelinTarkemmatTiedot.Create(XML: String);
begin
  VastaanotettavanMuodostaja(XML);
end;

constructor TAnnaPelinTarkemmatTiedot.Create;
begin
  LahetettavanMuodostaja(AnnaPelinTarkemmatTiedotTyyppi);
  XMLTulkki.SaveToXML(Paketti);
end;

constructor TPelinTarkemmatTiedot.Create(XML: String);
var JuuriNode, PelaajatNode, PelaajaNode: IXMLNode;
    I: Integer;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);

  MaksimiPelaajaMaara := StrToInt(JuuriNode.ChildNodes['maxpelaajia'].Text);
  Arvosana := StrToInt(JuuriNode.ChildNodes['arvosana'].Text);
  ArvosananKysyminen := StrToInt(JuuriNode.ChildNodes['arvosanankysyminen'].Text) <> 0;
  SalasanallaSuojattu := StrToInt(JuuriNode.ChildNodes['suojattu'].Text) <> 0;

  PelaajatNode := JuuriNode.ChildNodes['pelaajat'];
  PelaajienMaara := PelaajatNode.ChildNodes.Count;

  PelaajienNimet := TStringList.Create;
  PelaajienTunnukset := TStringList.Create;

  for I := 0 to PelaajienMaara - 1 do
  begin
     PelaajaNode := PelaajatNode.ChildNodes.Get(I);
     PelaajienNimet.Add(PelaajaNode.ChildNodes['nimi'].Text);
     PelaajienTunnukset.Add(PelaajaNode.ChildNodes['tunnus'].Text);
  end;

  PelinNimi := JuuriNode.ChildNodes['pelinnimi'].Text;
end;

constructor TPelinTarkemmatTiedot.Create(KMaksimiPelaajaMaara: Byte; KArvosana: Byte;
                   KArvosananKysyminen: Boolean; KSalasanallaSuojattu: Boolean;
                   KPelaajienNimet: TStringList; KPelaajienTunnukset: TStringList; KPelinNimi: String);
var JuuriNode, ApuNode, PelaajatNode, PelaajaNode: IXMLNode;
    I: Integer;
    PelaajienMaara: Integer;
begin
  JuuriNode := LahetettavanMuodostaja(PelinTarkemmatTiedotTyyppi);

  ApuNode := JuuriNode.AddChild('maxpelaajia');
  ApuNode.Text := IntToStr(KMaksimiPelaajaMaara);

  ApuNode := JuuriNode.AddChild('arvosana');
  ApuNode.Text := IntToStr(KArvosana);

  ApuNode := JuuriNode.AddChild('arvosanankysyminen');
  ApuNode.Text := IntToStr(Integer(KArvosananKysyminen));

  ApuNode := JuuriNode.AddChild('suojattu');
  ApuNode.Text := IntToStr(Integer(KSalasanallaSuojattu));

  PelaajienMaara := KPelaajienNimet.Count;
  
  PelaajatNode := JuuriNode.AddChild('pelaajat');

  for I := 0 to PelaajienMaara - 1 do
  begin
    PelaajaNode := PelaajatNode.AddChild('pelaaja');

    ApuNode := PelaajaNode.AddChild('nimi');
    ApuNode.Text := KPelaajienNimet[I];
    ApuNode := PelaajaNode.AddChild('tunnus');
    ApuNode.Text := KPelaajienTunnukset[I];
  end;

  ApuNode := JuuriNode.AddChild('pelinnimi');
  ApuNode.Text := KPelinNimi;

  XMLTulkki.SaveToXML(Paketti);
end;

destructor TPelinTarkemmatTiedot.Destroy;
begin
  PelaajienNimet.Free;
  PelaajienTunnukset.Free;
  inherited;
end;

constructor TAnnaPelaajalista.Create(XML: String);
begin
  VastaanotettavanMuodostaja(XML);
end;

constructor TAnnaPelaajalista.Create;
begin
  LahetettavanMuodostaja(AnnaPelaajalistaTyyppi);

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TPelaajalista.Create(XML: String);
var JuuriNode, PelaajaNode, PelaajatNode: IXMLNode;
    I, Pelaajia: Integer;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);

  PelaajatNode := JuuriNode.ChildNodes['pelaajat'];

  Pelaajia := PelaajatNode.ChildNodes.Count;

  Pelaajat := TStringList.Create;
  Tunnukset := TStringList.Create;

  for I := 0 to Pelaajia - 1 do
  begin
    PelaajaNode := PelaajatNode.ChildNodes.Get(I);

    Pelaajat.Add(PelaajaNode.ChildNodes['nimi'].Text);
    Pelaajat.Add(PelaajaNode.ChildNodes['tunnus'].Text);
  end;
end;

constructor TPelaajalista.Create(KPelaajat: TStringList; KTunnukset: TStringList);
var JuuriNode, ApuNode, PelaajaNode, PelaajatNode: IXMLNode;
    I: Integer;
begin
  JuuriNode := LahetettavanMuodostaja(PelaajalistaTyyppi);

  PelaajatNode := JuuriNode.AddChild('pelaajat');

  for I := 0 to KPelaajat.Count - 1 do
  begin
    PelaajaNode := PelaajatNode.AddChild('pelaaja');

    ApuNode := PelaajaNode.AddChild('nimi');
    ApuNode.Text := KPelaajat[I];

    ApuNode := PelaajaNode.AddChild('tunnus');
    ApuNode.Text := KTunnukset[I];
  end;

  XMLTulkki.SaveToXML(Paketti);
end;

destructor TPelaajalista.Destroy;
begin
  Pelaajat.Free;
  Tunnukset.Free;
  inherited;
end;

constructor TPelinTiedostot.Create(XML: String);
var JuuriNode, KysymyssarjatNode: IXMLNode;
    I, Kysymyssarjoja: Integer;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);

  Pelilauta := JuuriNode.ChildNodes['lauta'].Text;

  KysymyssarjatNode := JuuriNode.ChildNodes['kysymyssarjat'];

  Kysymyssarjoja := KysymyssarjatNode.ChildNodes.Count;

  Kysymyssarjat := TStringList.Create;

  for I := 0 to Kysymyssarjoja - 1 do
  begin
    Kysymyssarjat.Add(KysymyssarjatNode.ChildNodes.Get(I).Text);
  end;
end;

constructor TPelinTiedostot.Create(KPelilauta: String; KKysymyssarja: TStringList);
var JuuriNode, ApuNode, KysymyssarjatNode: IXMLNode;
    I, Kysymyssarjoja: Integer;
begin
  JuuriNode := LahetettavanMuodostaja(PelinTiedostotTyyppi);

  ApuNode := JuuriNode.AddChild('lauta');
  ApuNode.Text := KPelilauta;

  KysymyssarjatNode := JuuriNode.AddChild('kysymyssarjat');

  Kysymyssarjoja := KKysymyssarja.Count;

  for I := 0 to Kysymyssarjoja - 1 do
  begin
    ApuNode := KysymyssarjatNode.AddChild('kysymyssarja');
    ApuNode.Text := KKysymyssarja[I];
  end;

  XMLTulkki.SaveToXML(Paketti);
end;

destructor TPelinTiedostot.Destroy;
begin
  Kysymyssarjat.Free;
  inherited;
end;

constructor TLiityPeliin.Create(XML: String);
var JuuriNode: IXMLNode;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);

  PelaajanNimi := JuuriNode.ChildNodes['nimi'].Text;
  Tunnus := JuuriNode.ChildNodes['tunnus'].Text;
  SalasanaMD5 := StringToMD5(JuuriNode.ChildNodes['salasana'].Text);
end;

constructor TLiityPeliin.Create(KPelaajanNimi: String; KTunnus: String; KSalasanaMD5: MD5Digest);
var JuuriNode, ApuNode: IXMLNode;
begin
  JuuriNode := LahetettavanMuodostaja(LiityPeliinTyyppi);

  ApuNode := JuuriNode.AddChild('nimi');
  ApuNode.Text := KPelaajanNimi;
  ApuNode := JuuriNode.AddChild('tunnus');
  ApuNode.Text := KTunnus;
  ApuNode := JuuriNode.AddChild('salasana');
  ApuNode.Text := MD5Print(KSalasanaMD5);

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TLiityitPeliin.Create(XML: String);
var JuuriNode: IXMLNode;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);
  Hyvaksytty := Liittymiskoodi(StrToInt(JuuriNode.ChildNodes['hyvaksytty'].Text));
end;

constructor TLiityitPeliin.Create(KHyvaksytty: Liittymiskoodi);
var JuuriNode, ApuNode: IXMLNode;
begin
  JuuriNode := LahetettavanMuodostaja(LiityitPeliinTyyppi);
  ApuNode := JuuriNode.AddChild('hyvaksytty');
  ApuNode.Text := IntToStr(Integer(KHyvaksytty));

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TLiittyiPeliin.Create(XML: String);
var JuuriNode: IXMLNode;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);

  PelaajanNimi := JuuriNode.ChildNodes['nimi'].Text;
  Tunnus := JuuriNode.ChildNodes['tunnus'].Text;
end;

constructor TLiittyiPeliin.Create(KPelaajanNimi: String; KTunnus: String);
var JuuriNode, ApuNode: IXMLNode;
begin
  JuuriNode := LahetettavanMuodostaja(LiittyiPeliinTyyppi);

  ApuNode := JuuriNode.AddChild('nimi');
  ApuNode.Text := KPelaajanNimi;
  ApuNode := JuuriNode.AddChild('tunnus');
  ApuNode.Text := KTunnus;

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TPoistu.Create(XML: String);
begin
  VastaanotettavanMuodostaja(XML);
end;

constructor TPoistu.Create();
begin
  LahetettavanMuodostaja(PoistuTyyppi);

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TPoistui.Create(XML: String);
var JuuriNode: IXMLNode;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);

  PelaajanNimi := JuuriNode.ChildNodes['nimi'].Text;
  Tunnus := JuuriNode.ChildNodes['tunnus'].Text;
end;

constructor TPoistui.Create(KPelaajanNimi: String; KTunnus: String);
var JuuriNode, ApuNode: IXMLNode;
begin
  JuuriNode := LahetettavanMuodostaja(PoistuiTyyppi);

  ApuNode := JuuriNode.AddChild('nimi');
  ApuNode.Text := KPelaajanNimi;
  ApuNode := JuuriNode.AddChild('tunnus');
  ApuNode.Text := KTunnus;

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TPeliAlkaa.Create(XML: String);
begin
  VastaanotettavanMuodostaja(XML);
end;

constructor TPeliAlkaa.Create();
begin
  LahetettavanMuodostaja(PeliAlkaaTyyppi);

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TKenenVuoro.Create(XML: String);
var JuuriNode: IXMLNode;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);

  PelaajanNumero := Byte(StrToInt(JuuriNode.ChildNodes['pelaajannumero'].Text));
end;

constructor TKenenVuoro.Create(KPelaajanNumero:Byte);
var JuuriNode, ApuNode: IXMLNode;
begin
  JuuriNode :=  LahetettavanMuodostaja(KenenVuoroTyyppi);

  ApuNode := JuuriNode.AddChild('pelaajannumero');
  ApuNode.Text := IntToStr(KPelaajanNumero);

  XMLTulkki.SaveToXML(Paketti);
end;

constructor THeita.Create(XML: String);

begin
  VastaanotettavanMuodostaja(XML);
end;

constructor THeita.Create;
begin
  LahetettavanMuodostaja(HeitaTyyppi);

  XMLTulkki.SaveToXML(Paketti);
end;

constructor THeitonTulos.Create(XML: String);
var JuuriNode: IXMLNode;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);
  HeitonTulos := Byte(StrToInt(JuuriNode.ChildNodes['heitontulos'].Text));
  Siirtyma := Shortint(StrToInt(JuuriNode.ChildNodes['siirtyma'].Text));
end;

constructor THeitonTulos.Create(KHeitonTulos: Byte; KSiirtyma: Shortint);
var JuuriNode, ApuNode: IXMLNode;
begin
  JuuriNode := LahetettavanMuodostaja(HeitonTulosTyyppi);
  ApuNode := JuuriNode.AddChild('heitontulos');
  ApuNode.Text := IntToStr(KHeitonTulos);
  ApuNode := JuuriNode.AddChild('siirtyma');
  ApuNode.Text := IntToStr(KSiirtyma);

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TSiirretty.Create(XML: String);
var JuuriNode: IXMLNode;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);

  Siirtyma := ShortInt(StrToInt(JuuriNode.ChildNodes['siirtyma'].Text));
end;

constructor TSiirretty.Create(KSiirtyma: Shortint);
var JuuriNode, ApuNode: IXMLNode;
begin
  JuuriNode := LahetettavanMuodostaja(SiirrettyTyyppi);

  ApuNode := JuuriNode.AddChild('siirtyma');
  ApuNode.Text := IntToStr(KSiirtyma);

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TOdota.Create(XML: String);
begin
  VastaanotettavanMuodostaja(XML);
end;

constructor TOdota.Create;
begin
  LahetettavanMuodostaja(OdotaTyyppi);

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TKysymys.Create(XML: String);
var JuuriNode: IXMLNode;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);
  KysymyssarjanNumero := Byte(StrToInt(JuuriNode.ChildNodes['kysymyssarja'].Text));
  KysymyksenNumero := Word(StrToInt(JuuriNode.ChildNodes['kysymys'].Text));
end;

constructor TKysymys.Create(KKysymyssarjanNumero: Byte; KKysymyksenNumero: Word);
var JuuriNode, ApuNode: IXMLNode;
begin
  JuuriNode := LahetettavanMuodostaja(KysymysTyyppi);

  ApuNode := JuuriNode.AddChild('kysymyssarja');
  ApuNode.Text := IntToStr(KKysymyssarjanNumero);

  ApuNode := JuuriNode.AddChild('kysymys');
  ApuNode.Text := IntToStr(KKysymyksenNumero);

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TVastaus.Create(XML: String);
var JuuriNode: IXMLNode;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);
  Vastaus := JuuriNode.ChildNodes['vastaus'].Text;
end;

constructor TVastaus.Create(KVastaus: String; Lahetettava: Boolean);
var JuuriNode, ApuNode: IXMLNode;
begin
  JuuriNode := LahetettavanMuodostaja(VastausTyyppi);

  ApuNode := JuuriNode.AddChild('vastaus');
  ApuNode.Text := KVastaus;

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TTulos.Create(XML: String);
var JuuriNode: IXMLNode;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);
  Reaktio := Reaktiokoodi(StrToInt(JuuriNode.ChildNodes['reaktio'].Text));
end;

constructor TTulos.Create(KReaktio: Reaktiokoodi);
var JuuriNode, ApuNode: IXMLNode;
begin
  JuuriNode := LahetettavanMuodostaja(TulosTyyppi);
  ApuNode := JuuriNode.ChildNodes['reaktio'];
  ApuNode.Text := IntToStr(Integer(KReaktio));

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TPeliPaattyi.Create(XML: String);
var I, Maara: Integer;
    JuuriNode, ApuNode, PelaajatNode: IXMLNode;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);

  PelaajatNode := JuuriNode.ChildNodes['pelaajat'];

  Maara := PelaajatNode.ChildNodes.Count;

  PelaajienTunnukset := TStringList.Create;

  for I := 0 to Maara - 1 do
  begin
    ApuNode := PelaajatNode.ChildNodes.Get(I);
    PelaajienTunnukset.Add(ApuNode.Text);
  end;
end;

constructor TPeliPaattyi.Create(KPelaajienTunnukset: TStringList);
var I, Maara: Integer;
    JuuriNode, ApuNode, PelaajatNode: IXMLNode;
begin
  JuuriNode := LahetettavanMuodostaja(PeliPaattyiTyyppi);

  Maara := KPelaajienTunnukset.Count;

  PelaajatNode := JuuriNode.AddChild('pelaajat');

  for I := 0 to Maara - 1 do
  begin
    ApuNode := PelaajatNode.AddChild('tunnus');
    ApuNode.Text := KPelaajienTunnukset.Strings[I];
  end;

  XMLTulkki.SaveToXML(Paketti);
end;

destructor TPeliPaattyi.Destroy;
begin
  PelaajienTunnukset.Free;
  inherited;
end;

constructor TArvostelu.Create(XML: String);
var JuuriNode: IXMLNode;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);
  Arvosana := Byte(StrToInt(JuuriNode.ChildNodes['arvosana'].Text));
end;

constructor TArvostelu.Create(KArvosana: Byte);
var JuuriNode, ApuNode: IXMLNode;
begin
  JuuriNode := LahetettavanMuodostaja(ArvosteluTyyppi);

  ApuNode := JuuriNode.AddChild('arvosana');
  ApuNode.Text := IntToStr(KArvosana);

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TPeliPaattynyt.Create(XML: String);
var JuuriNode, ApuNode, ArvosanaNode: IXMLNode;
    I, Maara: Integer;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);
  Tilanne := Byte(StrToInt(JuuriNode.ChildNodes['tilanne'].Text));
  LoginNimi := JuuriNode.ChildNodes['loginnimi'].Text;
  PelilaudanNimi := JuuriNode.ChildNodes['pelilaudannimi'].Text;
  KysymyssarjojenNimet := TStringList.Create;
  ApuNode := JuuriNode.ChildNodes['kysymyssarjat'];
  Maara := ApuNode.ChildNodes.Count;
  for I := 0 to Maara - 1 do
  begin
    KysymyssarjojenNimet.Add(ApuNode.ChildNodes.Get(I).Text);
  end;
  ArvosanaTunnukset := TStringList.Create;
  Arvosanat := TStringList.Create;
  ApuNode := JuuriNode.ChildNodes['arvosanat'];
  Maara := ApuNode.ChildNodes.Count;
  for I := 0 to Maara - 1 do
  begin
    ArvosanaNode := ApuNode.ChildNodes.Get(I);
    ArvosanaTunnukset.Add(ArvosanaNode.ChildNodes['tunnus'].Text);
    Arvosanat.Add(ArvosanaNode.ChildNodes['arvosana'].Text);
  end;
end;

constructor TPeliPaattynyt.Create(KTilanne: Byte; KLoginNimi: String; KPelilaudanNimi: String;
                                   KKysymyssarjojenNimet: TStringList; KArvosanaTunnukset: TStringList;
                                   KArvosanat: TStringList);
var JuuriNode, ApuNode, ArvosanatNode, KysymyssarjaNode, ArvosanatiedotNode: IXMLNode;
    I, Maara: Integer;

begin
  JuuriNode := LahetettavanMuodostaja(PeliPaattynytTyyppi);
  ApuNode := JuuriNode.AddChild('tilanne');
  ApuNode.Text := IntToStr(KTilanne);
  ApuNode := JuuriNode.AddChild('loginnimi');
  ApuNode.Text := KLoginNimi;
  ApuNode := JuuriNode.AddChild('pelilaudannimi');
  ApuNode.Text := KPelilaudanNimi;

  Maara := KKysymyssarjojenNimet.Count;
  ApuNode := JuuriNode.AddChild('kysymyssarjat');
  for I := 0 to Maara - 1 do
  begin
    KysymyssarjaNode := ApuNode.AddChild('kysymyssarja');
    KysymyssarjaNode.Text := KKysymyssarjojenNimet[I];
  end;

  Maara := KArvosanaTunnukset.Count;
  ArvosanatNode := JuuriNode.AddChild('arvosanat');
  for I := 0 to Maara - 1 do
  begin
    ArvosanatiedotNode := ArvosanatNode.AddChild('arvosanatiedot');
    ApuNode := ArvosanatiedotNode.AddChild('tunnus');
    ApuNode.Text := KArvosanaTunnukset[I];
    ApuNode := ArvosanatiedotNode.AddChild('arvosana');
    ApuNode.Text := KArvosanat[I];
  end;

  XMLTulkki.SaveToXML(Paketti);
end;

destructor TPeliPaattynyt.Destroy;
begin
  KysymyssarjojenNimet.Free;
  ArvosanaTunnukset.Free;
  Arvosanat.Free;
  inherited;
end;

constructor TKatkaiseYhteys.Create(XML: String);
begin
  VastaanotettavanMuodostaja(XML);
end;

constructor TKatkaiseYhteys.Create;
begin
  LahetettavanMuodostaja(KatkaiseYhteysTyyppi);

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TVirhePaketti.Create(XML: String);
var JuuriNode: IXMLNode;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);

  Virhe := Virhekoodi(StrToInt(JuuriNode.ChildNodes['virhe'].Text));
end;
constructor TVirhePaketti.Create(KVirhe: Virhekoodi);
var JuuriNode, ApuNode: IXMLNode;
begin
  JuuriNode := LahetettavanMuodostaja(VirhePakettiTyyppi);

  ApuNode := JuuriNode.AddChild('virhe');
  ApuNode.Text := IntToStr(Integer(KVirhe));

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TKirjaaTiedostopalvelimeksi.Create(XML: String);
begin
  VastaanotettavanMuodostaja(XML);
end;

constructor TKirjaaTiedostopalvelimeksi.Create;
begin
  LahetettavanMuodostaja(KirjaaTiedostopalvelimeksiTyyppi);

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TAnnaTunnuslista.Create(XML: String);
begin
  VastaanotettavanMuodostaja(XML);
end;

constructor TAnnaTunnuslista.Create;
begin
  LahetettavanMuodostaja(AnnaTunnuslistaTyyppi);

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TTunnusLista.Create(XML: String);
var JuuriNode, TunnuksetNode, SalasanatNode, ApuNode: IXMLNode;
    I: Integer;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);

  Tunnukset := TStringList.Create;
  SalasanatMD5 := TStringList.Create;

  TunnuksetNode := JuuriNode.ChildNodes['tunnukset'];
  for I := 0 to TunnuksetNode.ChildNodes.Count - 1 do
  begin
    ApuNode := TunnuksetNode.ChildNodes.Get(I);
    Tunnukset.Add(ApuNode.Text);
  end;

  SalasanatNode := JuuriNode.ChildNodes['salasanat'];
  for I := 0 to SalasanatNode.ChildNodes.Count - 1 do
  begin
    ApuNode := SalasanatNode.ChildNodes.Get(I);
    SalasanatMD5.Add(ApuNode.Text);
  end;
end;

constructor TTunnusLista.Create(KTunnukset: TStringList; KSalasanatMD5: TStringList);
var JuuriNode, TunnuksetNode, SalasanatNode, ApuNode: IXMLNode;
    I: Integer;
begin
  JuuriNode := LahetettavanMuodostaja(TunnusListaTyyppi);

  TunnuksetNode := JuuriNode.AddChild('tunnukset');
  for I := 0 to KTunnukset.Count - 1 do
  begin
    ApuNode := TunnuksetNode.AddChild('tunnus');
    ApuNode.Text := KTunnukset[I];
  end;

  SalasanatNode := JuuriNode.AddChild('salasanat');
  for I := 0 to KSalasanatMD5.Count - 1 do
  begin
    ApuNode := SalasanatNode.AddChild('salasana');
    ApuNode.Text := KSalasanatMD5[I];
  end;

  XMLTulkki.SaveToXML(Paketti);
end;

destructor TTunnusLista.Destroy;
begin
  Tunnukset.Free;
  SalasanatMD5.Free;

  inherited;
end;

constructor TLisattyTunnus.Create(XML: String);
var JuuriNode: IXMLNode;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);

  Tunnus := JuuriNode.ChildNodes['tunnus'].Text;
  SalasanaMD5 := JuuriNode.ChildNodes['salasana'].Text;
end;

constructor TLisattyTunnus.Create(KTunnus: String; KSalasanaMD5: String);
var JuuriNode, ApuNode: IXMLNode;
begin
  JuuriNode := LahetettavanMuodostaja(LisattyTunnusTyyppi);

  ApuNode := JuuriNode.AddChild('tunnus');
  ApuNode.Text := KTunnus;

  ApuNode := JuuriNode.AddChild('salasana');
  ApuNode.Text := KSalasanaMD5;

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TAnnaTiedostopalvelimenTiedot.Create(XML: String);
begin
  VastaanotettavanMuodostaja(XML);
end;

constructor TAnnaTiedostopalvelimenTiedot.Create;
begin
  LahetettavanMuodostaja(AnnaTiedostopalvelimenTiedotTyyppi);

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TTiedostopalvelimenTiedot.Create(XML: String);
var JuuriNode: IXMLNode;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);

  Osoite := JuuriNode.ChildNodes['osoite'].Text;
  Portti := StrToInt(JuuriNode.ChildNodes['portti'].Text);
end;

constructor TTiedostopalvelimenTiedot.Create(KOsoite: String; KPortti: Integer);
var JuuriNode, ApuNode: IXMLNode;
begin
  JuuriNode := LahetettavanMuodostaja(TiedostopalvelimenTiedotTyyppi);

  ApuNode := JuuriNode.AddChild('osoite');
  ApuNode.Text := KOsoite;
  ApuNode := JuuriNode.AddChild('portti');
  ApuNode.Text := IntToStr(KPortti);

  XMLTulkki.SaveToXML(Paketti);
end;

constructor TPaketti.Create(XML: String);
begin
  VastaanotettavanMuodostaja(XML);
  Paketti := XML;
end;

{ TChattaajaLista }

constructor TChattaajaLista.Create(XML: String);
var JuuriNode, PelaajatNode, ApuNode: IXMLNode;
    I: Integer;
begin
  JuuriNode := VastaanotettavanMuodostaja(XML);

  PelaajienNimet := TStringList.Create;

  PelaajatNode := JuuriNode.ChildNodes['pelaajat'];
  for I := 0 to PelaajatNode.ChildNodes.Count - 1 do
  begin
    ApuNode := PelaajatNode.ChildNodes.Get(I);
    PelaajienNimet.Add(ApuNode.Text);
  end;
end;

constructor TChattaajaLista.Create(KPelaajienNimet: TStringList);
var JuuriNode, PelaajatNode, ApuNode: IXMLNode;
    I: Integer;
begin
  JuuriNode := LahetettavanMuodostaja(ChattaajaListaTyyppi);

  PelaajatNode := JuuriNode.AddChild('pelaajat');
  for I := 0 to KPelaajienNimet.Count - 1 do
  begin
    ApuNode := PelaajatNode.AddChild('nimi');
    ApuNode.Text := KPelaajienNimet[I];
  end;

  XMLTulkki.SaveToXML(Paketti);
end;

destructor TChattaajaLista.Destroy;
begin
  PelaajienNimet.Free;

  inherited;
end;

{ TAnnaChattaajaLista }

constructor TAnnaChattaajaLista.Create(XML: String);
begin
  VastaanotettavanMuodostaja(XML);
end;

constructor TAnnaChattaajaLista.Create;
begin
  LahetettavanMuodostaja(AnnaChattaajaListaTyyppi);

  XMLTulkki.SaveToXML(Paketti);
end;

end.

